from enums.Extension import Extension
from enums.Column import Column
from enums.Args import Args
from enums.NavigationalStatus import NavigationalStatus
from pathlib import Path
from polars import read_csv
from polars import col

class Processing(object):
    def __init__(self, path):
        # Transform the given path in a list to handle string, list, tuple or set
        pathList = list()
        pathList.append(path)

        # Handle the dataFrames for processing
        self.dataFrames = list()

        # Handle the names for the saving
        self.names = list()

        for path in pathList:
            if (isinstance(path, str)):
                path = Path(path)
                for extension in Extension:
                    if (path.is_file() and path.suffix == extension.value):
                        self.names.append(path.name)
                        self.dataFrames.append(read_csv(path, ignore_errors = Args.IGNORE_ERRORS.value, sep = Args.SEPARATOR.value))
                    elif (path.is_dir()):
                        for file in path.glob(Args.WILDCARD.value + extension.value):
                            self.names.append(file.name)
                            self.dataFrames.append(read_csv(file, ignore_errors = Args.IGNORE_ERRORS.value, sep = Args.SEPARATOR.value))
                    else:
                        print("The path \"" + str(path) + "\" is not a valid path. It will not be processed.")
                self.verifyColumns()
            else:
                print("The path \"" + str(path) + "\" is not a string. It will not be processed.")

    def verifyColumns(self):
        # Since CSV are all the same, we know the order of columns
        # but sometimes there is some CSV without columns names
        # this function will add the columns names if missing
        columns = list()
        for column in Column:
            columns.append(column.value)
        for i in range(len(self.dataFrames)):
            if (self.dataFrames[i].columns != columns):
                self.dataFrames[i].columns = columns

    def sort(self, column):
        for i in range(len(self.dataFrames)):
            self.dataFrames[i] = self.dataFrames[i].sort(column)

    def route(self):
        self.sort(Column.MMSI.value)
        for i in range (len(self.dataFrames)):
            self.dataFrames[i] = self.dataFrames[i].filter(

            # The first row can't be shifted since there is no data to compare,
            # so include the first row in the filter
            (col(Column.NAVIGATION_STATUS.value).is_first()) |

            # See if the MMSI changed between current and next row
            (col(Column.MMSI.value) != col(Column.MMSI.value).shift()) |

            # See if the navigation status changed between current and next row
            (col(Column.NAVIGATION_STATUS.value) != col(Column.NAVIGATION_STATUS.value).shift()))
            
        self.sort(Column.COMPLETE_SYS_DATE.value)

    def save(self, path):
        for i in range(len(self.dataFrames)):
            self.dataFrames[i].to_csv(path + self.names[i], sep = ";")

l = "C:\\Users\\jerem\\Documents\\FAC\\M1\\TPE\\ais-operations\\ressources\\BD-AIS\\FichiersCSV-Exemples\\aishub-data-20161220.csv"
test = Processing(l)
test.verifyColumns()
#test.route()
#test.save("C:\\Users\\jerem\\Documents\\FAC\\M1\\TPE\\ais-operations\\resources\\")